package GCDisplay;

###################################################
#
#  Copyright 2005-2006 Tian
#
#  This file is part of GCstar.
#
#  GCstar is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  GCstar is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with GCstar; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
###################################################

use strict;
use GCUtils;

my $firstYear = 1895;

{
    package GCFilterSearch;

    sub compareExact
    {
        my ($field, $value) = @_;
        return $field eq $value;
    }
    
    sub compareContain
    {
        my ($field, $value) = @_;
        return $field =~ m/$value/i;
    }

    sub compareLessStrings
    {
        my ($field, $value) = @_;
        return $field lt $value;
    }

    sub compareLessNumbers
    {
        my ($field, $value) = @_;
        return 0 if !defined($field);
        return $field < $value;
    }

    sub compareLessOrEqualStrings
    {
        my ($field, $value) = @_;
        return $field le $value;
    }

    sub compareLessOrEqualNumbers
    {
        my ($field, $value) = @_;
        return 0 if !defined($field);
        return $field <= $value;
    }
    
    sub compareGreaterStrings
    {
        my ($field, $value) = @_;
        return $field gt $value;
    }

    sub compareGreaterNumbers
    {
        my ($field, $value) = @_;
        return 0 if !defined($field);
        return $field > $value;
    }

    sub compareGreaterOrEqualStrings
    {
        my ($field, $value) = @_;
        return $field ge $value;
    }

    sub compareGreaterOrEqualNumbers
    {
        my ($field, $value) = @_;
        return 0 if !defined($field);
        return $field >= $value;
    }

    sub compareRangeStrings
    {
        my ($field, $value) = @_;
        my @values = split m/;/, $value;
        return ($field ge $values[0]) && ($field le $values[1]);
    }

    sub compareRangeNumbers
    {
        my ($field, $value) = @_;
        return 0 if !defined($field);
        my @values = split m/;/, $value;
        return ($field >= $values[0]) && ($field <= $values[1]);
    }

    sub new
    {
        my ($proto, $info) = @_;
        my $class = ref($proto) || $proto;
        my $self  = {};

        bless ($self, $class);

        $self->{info} = $info if $info;
        $self->clear if !$info;

        return $self;
    }

    sub clear
    {
        my $self = shift;
        
        $self->{cleared} = 1;

        $self->{info} = {};
    }
    
    sub getComparisonFunction
    {
        my ($self, $type) = @_;
        
        my ($comparison, $numeric) = @$type;
        if ($comparison eq 'eq')
        {
            return \&compareExact;
        }
        elsif ($comparison eq 'contain')
        {
            return \&compareContain;
        }
        elsif ($comparison eq 'lt')
        {
            return \&compareLessStrings if ($numeric ne 'true');
            return \&compareLessNumbers if ($numeric eq 'true');
        }
        elsif ($comparison eq 'le')
        {
            return \&compareLessOrEqualStrings if ($numeric ne 'true');
            return \&compareLessOrEqualNumbers if ($numeric eq 'true');
        }
        elsif ($comparison eq 'gt')
        {
            return \&compareGreaterStrings if ($numeric ne 'true');
            return \&compareGreaterNumbers if ($numeric eq 'true');
        }
        elsif ($comparison eq 'ge')
        {
            return \&compareGreaterOrEqualStrings if ($numeric ne 'true');
            return \&compareGreaterOrEqualNumbers if ($numeric eq 'true');
        }
        elsif ($comparison eq 'range')
        {
            return \&compareRangeStrings if ($numeric ne 'true');
            return \&compareRangeNumbers if ($numeric eq 'true');
        }
    }
    
    sub setFilter
    {
        my ($self, $filter, $value, $type, $model) = @_;
        if (!$filter)
        {
            $self->clear;
            return;
        }
        if ($value eq '')
        {
            delete $self->{info}->{$filter};
        }
        else
        {
            $self->{cleared} = 0;
            my $preprocess = $type->[2];
            if (!$preprocess)
            {
                my $fieldType = $model->{fieldsInfo}->{$filter}->{type};
                $preprocess = ($fieldType eq 'date') ? 'reverseDate'
                            : ($fieldType eq 'single list') ? 'singleList'
                            : ($fieldType eq 'double list') ? 'doubleList'
                            : ($fieldType =~ /list$/o) ? 'singleList'
                            : '';
            }
            $self->{info}->{$filter} = {
                                        value => $value,
                                        comp => $self->getComparisonFunction($type),
                                        preprocess => $preprocess
                                       };
        }
    }

    sub setModel
    {
        my ($self, $model) = @_;
        
        $self->{model} = $model;
    }

    sub test
    {
        my ($self, $info) = @_;
        return 1 if $self->{cleared};
        foreach my $field(keys %{$self->{info}})
        {
            my $value = $info->{$field};
            if ($self->{info}->{$field}->{preprocess})
            {
                my $preProcess = 'GCPreProcess::'.$self->{info}->{$field}->{preprocess};
                eval {
                    no strict qw/refs/;
                    $value = $preProcess->($value);
                };
            }
            return 0 if $self->{info}->{$field}
                     && ! $self->{info}->{$field}->{comp}->($value, $self->{info}->{$field}->{value});
        }
        return 1;
    }
}

{
    package GCFilterBorrowing;

    sub new
    {
        my ($proto, $borrowed) = @_;
        my $class = ref($proto) || $proto;
        my $self  = {};
        
        $self->{borrowed} = $borrowed;
        
        bless ($self, $class);
        return $self;
    }

    sub test
    {
        my ($self, $info) = @_;

        return (($info->{borrower} eq 'none') || !$info->{borrower}) 
            if ! $self->{borrowed};
        return (($info->{borrower} ne 'none') && $info->{borrower}) 
    }
}

use Gtk2;

{
    package GCSearchDialog;
    
    use GCGraphicComponents;
    
    use base "GCModalDialog";
    
    sub initValues
    {
        my $self = shift;
        
        my $info = $self->{parent}->{items}->{filterSearch}->{info};
        
        foreach (@{$self->{fields}})
        {
            if (exists $info->{$_})
            {
                $self->{$_}->setValue($info->{$_}->{value});
            }
            else
            {
                $self->{$_}->clear if $self->{$_};
            }
            if ($self->{fieldsInfo}->{$_}->{type} eq 'history text')
            {
                $self->{$_}->setValues($self->{parent}->{panel}->{$_}->getValues);
            }
            if (
                 (
                   ($self->{fieldsInfo}->{$_}->{type} eq 'single list')
                   ||
                   ($self->{fieldsInfo}->{$_}->{type} eq 'double list')
                 )
                 &&
                 (
                   $self->{parent}->{panel}->{$_}->{withHistory}
                 )
               )
            {
                my @values;
                foreach ($self->{parent}->{panel}->{$_}->getValues)
                {
                    push @values, $_->[0];
                }
                $self->{$_}->setValues(@values);
            }
        }
    }
    
    sub show
    {
        my $self = shift;

        $self->initValues;
        
        $self->SUPER::show();
        $self->show_all;
        $self->{search} = undef;
        my $ended = 0;
        while (!$ended)
        {
            my $response = $self->run;
            if ($response eq 'ok')
            {
                my %info;
            
                foreach (@{$self->{fields}})
                {
                    $info{$_} = $self->{$_}->getValue
                        if ! $self->{$_}->isEmpty;
                }

                $self->{parent}->{menubar}->initFilters(\%info);

                $self->{search} = \%info;
            }
            $ended = 1 if ($response eq 'ok') || ($response eq 'cancel') || ($response eq 'delete-event');
            $self->clear if ($response eq 'reject');
        }
        $self->hide;
    }
    
    sub clear
    {
        my $self = shift;

        foreach (@{$self->{fields}})
        {
            $self->{$_}->clear;
        }
    }
    
    sub search
    {
        my $self = shift;
        
        return $self->{search};
    }
    
    sub new
    {
        my ($proto, $parent) = @_;
        my $class = ref($proto) || $proto;
        my $self  = $class->SUPER::new($parent,
                                       $parent->{lang}->{SearchTitle},
                                       Gtk2::Stock->lookup('gtk-find')->{label},
                                       0,
                                       'gtk-clear' => 'reject',
                                      );
        bless ($self, $class);

        $self->{parent} = $parent;
        $self->{layoutTable} = new Gtk2::Table(1,4,0);
        $self->{layoutTable}->set_row_spacings($GCUtils::halfMargin);
        $self->{layoutTable}->set_col_spacings($GCUtils::margin);
        $self->{layoutTable}->set_border_width($GCUtils::margin);
        
        $self->vbox->pack_start($self->{layoutTable}, 1, 1, 0);
        return $self;
    }
    
    sub setModel
    {
        my ($self, $model) = @_;

        foreach ($self->{layoutTable}->get_children)
        {
            $self->{layoutTable}->remove($_);
            $_->destroy;
        }
        #$self->{layoutTable}->set_size_request(-1,-1);

        my $fieldsInfo = $model->{fieldsInfo};
        $self->{fieldsInfo} = $fieldsInfo;
        my @filtersGroup = @{$model->{filtersGroup}};
        my @filtersTotal = @{$model->{filters}};

        my $row = 0;
        return if ($#filtersTotal + (2 * $#filtersGroup)) <= 0;
        $self->{layoutTable}->resize($#filtersTotal + (2 * $#filtersGroup),4);

        $self->{fields} = [];
        foreach my $group(@filtersGroup)
        {
            $row++;
            my @filters = @{$group->{filter}};
            my $label = new GCHeaderLabel($model->getDisplayedText($group->{label}));
            $self->{layoutTable}->attach($label, 0, 4, $row, $row + 1, 'fill', 'expand', 0, 0);
            $row++;
            foreach my $filter(@filters)
            {
                my $field = $filter->{field};
                if ($field ne 'separator')
                {
                    push @{$self->{fields}}, $field;
                    my $labelText = $fieldsInfo->{$field}->{displayed};
                    $labelText = $model->getDisplayedText($filter->{label}) if $filter->{label};            
                    my $label = new Gtk2::Label($labelText);
                    $label->set_alignment(0,0.5);
                    $self->{layoutTable}->attach($label, 2, 3, $row, $row + 1, 'fill', 'fill', 0, 0);
                    if ($fieldsInfo->{$field}->{type} eq 'short text')
                    {
                        if ($filter->{comparison} eq 'range')
                        {
                            $self->{$field} = new GCRange('text', $self->{parent}->{lang});
                            $self->{$field}->setWidth(16);
                            
                        }
                        else
                        {
                            $self->{$field} = new GCShortText;
                        }
                        $self->{$field}->signal_connect('activate' => sub {$self->response('ok')} );
                    }
                    elsif ($fieldsInfo->{$field}->{type} eq 'number')
                    {
                        #If we want to have values that are less to the specified one,
                        #we use max as default to be sure everything will be returned
                        #in that case.
                        my $default = $fieldsInfo->{$field}->{min};
                        $default = $fieldsInfo->{$field}->{max}
                            if $filter->{comparison} =~ /^l/;
                        if (exists $fieldsInfo->{$field}->{min})
                        {
                            if ($filter->{comparison} eq 'range')
                            {
                                $self->{$field} = new GCRange('number',
                                                              $self->{parent}->{lang},
                                                              $fieldsInfo->{$field}->{min},
                                                              $fieldsInfo->{$field}->{max},
                                                              $fieldsInfo->{$field}->{step});
                                $self->{$field}->setWidth(16);
                            }
                            else
                            {
                                $self->{$field} = new GCNumeric($default,
                                                                $fieldsInfo->{$field}->{min},
                                                                $fieldsInfo->{$field}->{max},
                                                                $fieldsInfo->{$field}->{step});
                            }
                        }
                        else
                        {
                            if ($filter->{comparison} eq 'range')
                            {
                                $self->{$field} = new GCRange('numeric text', $self->{parent}->{lang});
                                $self->{$field}->setWidth(16);
                            }
                            else
                            {
                                $self->{$field} = new GCNumericText;
                            }
                        }
                    }
                    elsif (($fieldsInfo->{$field}->{type} eq 'history text')
                        || (($fieldsInfo->{$field}->{type} =~ /list/)
                         && ($fieldsInfo->{$field}->{history} ne 'false')))
                    {
                        $self->{$field} = new GCHistoryText;
                    }
                    elsif ($fieldsInfo->{$field}->{type} eq 'options')
                    {
                        $self->{$field} = new GCMenuList;
                        $self->{$field}->setValues($model->getValues($fieldsInfo->{$field}->{'values'}), $fieldsInfo->{$field}->{separator});
                    }
                    elsif ($fieldsInfo->{$field}->{type} eq 'yesno')
                    {
                        $self->{$field} = new GCCheckBoxWithIgnore($self->{parent});
                    }
                    elsif ($fieldsInfo->{$field}->{type} eq 'date')
                    {
                        $self->{$field} = new GCDate($self, $self->{parent}->{lang}, 1);
                    }
                    else
                    {
                        $self->{$field} = new GCShortText;
                    }
                    $self->{layoutTable}->attach($self->{$field}, 3, 4, $row, $row + 1, 'fill', 'expand', 0, 0);
                }
                else
                {
                    $self->{layoutTable}->attach(Gtk2::HSeparator->new, 0, 2, $row, $row + 1, 'fill', 'fill', 0, 0);
                }
                $row++;
            }
        }
        #$self->resize(1,1);
    }
}


1;